%%
%     ---------------------------------------------------------------------
%     Covariogram-based CS data gathering & compression framework
%     ---------------------------------------------------------------------
%     Performance evaluation framework including: 
%               - generation of correlated spatio-temporal signals
%               - scheduling, sampling & data gathering
%               - signal compression (at the sensors) & reconstruction (at the sink)
%
%     Copyright (C) 2016 Mohsen Hooshmand, Davide Zordan, Michele Rossi
%     Department of Information Engineering, University of Padova, Italy
%     Email: {hooshmand, zordan, rossi}@dei.unipd.it
%     (this module was originally written by Giorgio Quer and Riccardo Masiero)
% 
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program in the file "gpl-license-3.0.txt".  
%
%     If not, see <http://www.gnu.org/licenses/>.
%% 
    
function [y,phi,index] = BLOCK4_network_cs_rns(x, p_tx)
%%%% BLOCK 4: Aggregation 
% INPUTS:
%        x [N * 1] = (values sensed by each node)
%        p_tx = probability of transmission for each node
% OUTPUTS:
%        y [vector] = values of the packets received by the sink (length=M)
%        phi [matrix M x N] = matrix s.t. y = Phi*x
%        index = indices of the sensors that sense the environment
%


% total number of nodes (sink excluded, the sink is not a sensing node)
N = length(x);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Random Sampling
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% choose randomly (and indipendently for each node)
index = find([rand(N,1)<=p_tx]);
M = length(index);


phi = zeros(M,N) ; % inizialization
for i = 1:length(index)
    phi(i,index(i)) = 1 ;
end;

% values (data) transmitted to the data collection point
y = phi*x;
end % function
