%%
%     ---------------------------------------------------------------------
%     Covariogram-based CS data gathering & compression framework
%     ---------------------------------------------------------------------
%     Performance evaluation framework including: 
%               - generation of correlated spatio-temporal signals
%               - scheduling, sampling & data gathering
%               - signal compression (at the sensors) & reconstruction (at the sink)
%
%     Copyright (C) 2016 Mohsen Hooshmand, Davide Zordan, Michele Rossi
%     Department of Information Engineering, University of Padova, Italy
%     Email: {hooshmand, zordan, rossi}@dei.unipd.it
%     (this module was originally written by Giorgio Quer and Riccardo Masiero)
% 
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program in the file "gpl-license-3.0.txt".  
%
%     If not, see <http://www.gnu.org/licenses/>.
%% 

function [y,phi] = BLOCK4_network_tdma(x, time_table, cur_time)
%%%% BLOCK 4: 
% INPUTS:
%        x [N * 1] = (values sensed by each node)
%        time_table = 0-1 matrix [N * T] that shows when each sensor have
%        to sense or not to      
%        cur_time = Current time step for the reconstruction
% OUTPUTS:
%        y [vector] = values of the packets received by the sink (length=M)
%        phi [matrix M x N] = matrix s.t. y = Phi*x
%

% total number of nodes (sink excluded, the sink is not a sensing node)
N = length(x);

set=squeeze(time_table(:,cur_time));
index = find(set==1);

% number of the transmitting nodes
M = length(index);

phi = zeros(M,N) ; % inizialization
for i = 1:length(index)
    phi(i,index(i)) = 1 ;
end;

% values (data) transmitted to the data collection point
y = phi*x;
end % function
