%%
%     ---------------------------------------------------------------------
%     Covariogram-based CS data gathering & compression framework
%     ---------------------------------------------------------------------
%     Performance evaluation framework including: 
%               - generation of correlated spatio-temporal signals
%               - scheduling, sampling & data gathering
%               - signal compression (at the sensors) & reconstruction (at the sink)
%
%     Copyright (C) 2016 Mohsen Hooshmand, Davide Zordan, Michele Rossi
%     Department of Information Engineering, University of Padova, Italy
%     Email: {hooshmand, zordan, rossi}@dei.unipd.it
%     (this module was originally written by Giorgio Quer and Riccardo Masiero)
% 
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program in the file "gpl-license-3.0.txt".  
%
%     If not, see <http://www.gnu.org/licenses/>.
%% 


function [y,phi_underline] = BLOCK4_network_tdma_kron(X, time_table, cur_time, W)
%%%% BLOCK 4: (by MH modified 02.11.15)
% INPUTS:
%        x [N * 1] = (values sensed by each node)
%        time_table = 0-1 matrix [N * T] that shows when each sensor have
%        to sense or not to      
%        cur_time = Current time step for the reconstruction
%        W = the number of time steps of the windows used for the temporal basis
%
%
% OUTPUTS:
%        y [vector] = values of the packets received by the sink (length=MW)
%        phi_underline [matrix MW x NW] = matrix s.t. y = phi_underline*x_underline
%

% total number of nodes (sink excluded, the sink is not a sensing node)
N = size(X,1);
X_underline = zeros(N*W,1);
set = squeeze(time_table(:,cur_time));
index = find(set==1);
M = length(index);
phi_underline = zeros(M*W,N*W) ; % inizialization
for i = 1: W
    X_underline((i-1)*N+1:i*N) = X(:,cur_time-W+i);
    set = squeeze(time_table(:,cur_time-W+i));
    index = find(set==1);
    M = length(index);
    phi = zeros(M,N) ;
    for j = 1:length(index)
        phi(j,index(j)) = 1 ;
    end
    phi_underline((i-1) * M + 1 : (i-1) * M + M, (i-1) * N + 1 : (i-1) * N + N) = phi;
end

% values (data) transmitted to the data collection point
y = phi_underline*X_underline;

end
