
%%
%     ---------------------------------------------------------------------
%     Covariogram-based CS data gathering & compression framework
%     ---------------------------------------------------------------------
%     Performance evaluation framework including: 
%               - generation of correlated spatio-temporal signals
%               - scheduling, sampling & data gathering
%               - signal compression (at the sensors) & reconstruction (at the sink)
%
%     Copyright (C) 2016 Mohsen Hooshmand, Davide Zordan, Michele Rossi
%     Department of Information Engineering, University of Padova, Italy
%     Email: {hooshmand, zordan, rossi}@dei.unipd.it
% 
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program in the file "gpl-license-3.0.txt".  
%
%     If not, see <http://www.gnu.org/licenses/>.
%% 
    
function [X_hat, tot_cost, real_error] = BLOCKmain_dsc(A, T, hops_vector, r_s, r_compressed, Cid, NoS, all_indeces)
%% BLOCK main DSC (by MH modified 02.11.15)
% INPUT:
%         A [N * T matrix] = whole signal sensed to be analyzed, composed
%             by T (time) signals of length N
%         T = time
%         cost_per_trasm [N vector] =
%             cost of transmission for each node
%        hops_vector = vector where each i-th element represents the number of
%        hops among the i-th sensor and the sink x---o--o--X (= 3 hops)
%        r_s =  complete smaple bitlength
%        r_compressed =  compressed smaple bitlength
%        Cid = cluster of each Sensor
%        NoS =  Number of sensors
%        all_indeces : bin Id corresponding to each code
%
%
% OUTPUT:
%         X_hat [N x T matrix] = whole signal reconstructured, composed
%             by T (time) signals of length N
%         tot_cost = enrgy consumption for transmission of the sensed
%         readings
%         real_error[1 x T] = vector with all the real errors of the net
%

X_hat=zeros(size(A));

trans_bit = zeros(NoS,1);%cost_per_trasm.*initial_training;
comp_cycle=zeros(NoS,1);
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%quantization
X=A;
highest_amplitude = max(max(X));
bias = min(min(X));
if(bias < 0)
    X = X - bias;
end
delta = (highest_amplitude - bias) / (2^r_s);

x_decimal = floor(X ./delta - 0.5);
x_decimal(x_decimal > (2^r_s-1))=2^r_s-1;
x_decimal(x_decimal < 0)=0;

% reshape_X=reshape(x_decimal,NoS*T,1);

%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%Encoder
% max(size(reshape_X))
for i=1:NoS
    for j=1:T
        rem_x_decimal(i,j)=rem(x_decimal(i,j),2^r_compressed);
    end
end
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%Decoder
for i=1:NoS
    for j=1:T
        [same_bin] = find(all_indeces==rem_x_decimal(i,j));
        
        diff=2^(r_s+1);
        counter=0;
        v1=x_decimal(Cid(i),j);
        for l=1:max(size(same_bin))
            v2=(same_bin(l)-1);
            dist=abs(v2-v1);   %all_indeces(same_bin(l)));
            if(dist<diff)
                counter= counter+1;
                diff=dist;
                Xindex_hat(i,j)=v2;
                X_hat(i,j)=Xindex_hat(i,j).*delta;
            end
        end
    end
end
%%

if(bias < 0)
    X_hat = X_hat + bias;
end
for i=1:NoS
    if(Cid(i)==i)
        X_hat(i,:)= A(i,:);
    end
end

%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%Energy Consumption
for k=1:NoS
    if(Cid(k)==k)
        trans_bit(k)=T*r_s;
        comp_cycle(k)=T.*(2*405);
    else
        trans_bit(k)=T*r_compressed;
        comp_cycle(k)=T.*(4*405);
    end
end
tot_cost=energy_consumption_header( hops_vector, [1:NoS], trans_bit )+sum(comp_cycle.*0.726);
real_error = mean(sqrt((X_hat - A).^2));
end
