
%%
%     ---------------------------------------------------------------------
%     Covariogram-based CS data gathering & compression framework
%     ---------------------------------------------------------------------
%     Performance evaluation framework including: 
%               - generation of correlated spatio-temporal signals
%               - scheduling, sampling & data gathering
%               - signal compression (at the sensors) & reconstruction (at the sink)
%
%     Copyright (C) 2016 Mohsen Hooshmand, Davide Zordan, Michele Rossi
%     Department of Information Engineering, University of Padova, Italy
%     Email: {hooshmand, zordan, rossi}@dei.unipd.it
% 
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program in the file "gpl-license-3.0.txt".  
%
%     If not, see <http://www.gnu.org/licenses/>.
%% 
    
function [ Cid, Index_o_HC, Belonging ] = clustering_dsc( NoS, SoF_X, alpha, TR, IX, Dist_mat )
%% Clustering for DSC (by MH modified 02.11.15)
% -- Function File:  

%     generating clusters for Distributed Source Coding and assigning sensors
%     to the corresponding clusters
%
%     Input:
%       NoS           : # of sensors
%       SoF_X         : Size of the Field
%       alpha         : scaling factor
%       TR            : radius of transmission Range
%       IX            : sensors' sorting based on highest number of neighbors 
%       Dist_mat      : Distance among sensors matrix
%
%     Output:
%       Cid             : cluster of each Sensor
%       Index_o_HC      : signal for random location S ---synt_dim*synt_dim*T
%       Belonging       : 0-1 vector sensors neighbor with sink

% Cluster Creation
Belonging=zeros(NoS,1); %  0: not assigned, 1: dead, 2: assigned, 3: HC
Cid=zeros(NoS,1); % Head cluster of each sensor
% Cluster head Selection
j=1;

for i=1:NoS
    Counter=1;
    for k=1:j-1
        if(Dist_mat(IX(i),Index_o_HC(k))<(alpha*TR) ) % Maybe correct
            Counter=0;
        end
    end
    if(Counter==1)
        Belonging(IX(i))=3; % 3: HC
        Index_o_HC(j)=IX(i);
        Cid(IX(i))=IX(i);
        j=j+1;
    end
end

NoCT=size(Index_o_HC);
NoC=NoCT(2);

% Node assigning to each Cluster

for i=1:NoS
    min_distance=SoF_X +10;
    min_index=0;
    if(Belonging(i)==0)
        for j=1:NoC
            if(Dist_mat(i,Index_o_HC(j))<min_distance)
                min_distance=Dist_mat(i,Index_o_HC(j));
                min_index=Index_o_HC(j);
            end
        end
        Belonging(i)=2;
        Cid(i)=min_index;
    end
end
end
