%%
%     ---------------------------------------------------------------------
%     Covariogram-based CS data gathering & compression framework
%     ---------------------------------------------------------------------
%     Performance evaluation framework including: 
%               - generation of correlated spatio-temporal signals
%               - scheduling, sampling & data gathering
%               - signal compression (at the sensors) & reconstruction (at the sink)
%
%     Copyright (C) 2016 Mohsen Hooshmand, Davide Zordan, Michele Rossi
%     Department of Information Engineering, University of Padova, Italy
%     Email: {hooshmand, zordan, rossi}@dei.unipd.it
% 
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program in the file "gpl-license-3.0.txt".  
%
%     If not, see <http://www.gnu.org/licenses/>.
%% 
    
function [model,dim,n_op] = dct_encoder_f(x,tol)

c_div = 0;
c_mul = 0;
c_sum = 0;
c_cmp = 0;

N = length(x);
X = dct(x);
c_sum = N/2*(3*(nextpow2(N))-2)+1;
c_mul = (nextpow2(N))*N/2;

stop = 0;

FIN = zeros(1,N);
dim = 0;

while ~stop
    dim = dim+1;

    FIN(dim) = X(dim); 

    final_x = idct(FIN);
    c_sum = c_sum +  N/2*(3*log2(nextpow2(N))-2)+1;
    c_mul = c_mul + log2(nextpow2(N))*N/2;
    
    err = max(abs(x - final_x));
    c_cmp = c_cmp + N + 1;
    c_sum = c_sum + N;
    c_div = c_div + 1;
  
    if (err<=tol) || (dim==N) 
        stop = 1;
    end

end

model = FIN(1:dim);
n_op = [c_div,c_mul,c_sum,c_cmp]; 
