
%%
%     ---------------------------------------------------------------------
%     Covariogram-based CS data gathering & compression framework
%     ---------------------------------------------------------------------
%     Performance evaluation framework including: 
%               - generation of correlated spatio-temporal signals
%               - scheduling, sampling & data gathering
%               - signal compression (at the sensors) & reconstruction (at the sink)
%
%     Copyright (C) 2016 Mohsen Hooshmand, Davide Zordan, Michele Rossi
%     Department of Information Engineering, University of Padova, Italy
%     Email: {hooshmand, zordan, rossi}@dei.unipd.it
% 
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program in the file "gpl-license-3.0.txt".  
%
%     If not, see <http://www.gnu.org/licenses/>.
%% 
    
function [ U ] = direct_eig_computation( Sigma )
%direct eig
%   compute the ordered eigenvaectors of the input matrix "Sigma" 

% [T,E] = eig(Sigma); 
[T,E] = svd(Sigma);
% keyboard
Td = double(T); % eigenvectors of Sigma

Ed = double(E); % diag(Ed) = eigenvalues of Sigma

[~, permutazione] = sort(diag(Ed), 'descend'); % eigenvalues of Sigma
% in descending order

U = permuta(Td,permutazione); % eigenvectors in descending order
end
%% function permuta
%INPUT:  matrix, a matrix to order according to a given sort vector
%        permutazione, the sort vector
%OUTPUT: matrix_p, the sorted matrix
function [matrix_p] = permuta(matrix,permutazione)
matrix_p=zeros(size(matrix));
for p = 1:length(permutazione)
    
    matrix_p(:,p) = matrix(:,permutazione(p));
    
end


end
