%%
%     ---------------------------------------------------------------------
%     Covariogram-based CS data gathering & compression framework
%     ---------------------------------------------------------------------
%     Performance evaluation framework including: 
%               - generation of correlated spatio-temporal signals
%               - scheduling, sampling & data gathering
%               - signal compression (at the sensors) & reconstruction (at the sink)
%
%     Copyright (C) 2016 Mohsen Hooshmand, Davide Zordan, Michele Rossi
%     Department of Information Engineering, University of Padova, Italy
%     Email: {hooshmand, zordan, rossi}@dei.unipd.it
% 
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program in the file "gpl-license-3.0.txt".  
%
%     If not, see <http://www.gnu.org/licenses/>.
%% 
    
function [ distance_vector , new_distances ] = distance_reducing( distances)
%distance_reducing  
% -- Function File:  

%     definig uniformed distances
%
%     Input:
%     distances     : Distance among sensors matrix
%
%     Output:
%     distance_vector           : uniformed distance between min and max of distances 
%     new_distances             : New distance matrix of sensors based on
%     the new distance_vector

NoS=length(distances);
min_distance = min(min(distances));
max_distance = max(max(distances));
distance_interval = (max_distance - min_distance)/(50-1);
distance_vector=min_distance:distance_interval:max_distance;
new_distances=zeros(NoS,NoS);
for ii=1:NoS
    for jj=1:NoS
        temp=abs(distance_vector-distances(ii,jj));
        [~, idx] = min(temp);
        new_distances(ii,jj) = distance_vector(idx);
    end
end


end

