
%%
%     ---------------------------------------------------------------------
%     Covariogram-based CS data gathering & compression framework
%     ---------------------------------------------------------------------
%     Performance evaluation framework including: 
%               - generation of correlated spatio-temporal signals
%               - scheduling, sampling & data gathering
%               - signal compression (at the sensors) & reconstruction (at the sink)
%
%     Copyright (C) 2016 Mohsen Hooshmand, Davide Zordan, Michele Rossi
%     Department of Information Engineering, University of Padova, Italy
%     Email: {hooshmand, zordan, rossi}@dei.unipd.it
% 
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program in the file "gpl-license-3.0.txt".  
%
%     If not, see <http://www.gnu.org/licenses/>.
%% 
    
function [ S ] = energy_consumption_header( hops_vector, indexes, packetsize_vector )
%%ENERGY CONSUMPTION returns a vector where each value contains the of energy consumption
%% (by MH modified 02.11.15)
% INPUTS:
%        hops_vector = vector where each i-th element represents the number of
%        hops among the i-th sensor and the sink x---o--o--X (= 3 hops)
%        indexes = ids of the sensors which are transmitting
%        packetsize_vector = vector containing the size of the data packets
%        sent by each sensor
% OUTPUTS:
%       S = a vector where each item represents the amount of consumption
%       energy for a sensor

packet_witheader=ceil(packetsize_vector./(960)).*104 + packetsize_vector;

S = sum(hops_vector([indexes]).*(rx_energy(packet_witheader([indexes]))+tx_energy(packet_witheader([indexes])))-rx_energy(packet_witheader([indexes])));

end

function [ rx_energy ] = rx_energy(packet_size) 
rx_energy = packet_size.*230; % 230nj/bit
end

function [ tx_energy ] = tx_energy(packet_size)
tx_energy = packet_size.*230;  % 230nj/bit
end
