
%%
%     ---------------------------------------------------------------------
%     Covariogram-based CS data gathering & compression framework
%     ---------------------------------------------------------------------
%     Performance evaluation framework including: 
%               - generation of correlated spatio-temporal signals
%               - scheduling, sampling & data gathering
%               - signal compression (at the sensors) & reconstruction (at the sink)
%
%     Copyright (C) 2016 Mohsen Hooshmand, Davide Zordan, Michele Rossi
%     Department of Information Engineering, University of Padova, Italy
%     Email: {hooshmand, zordan, rossi}@dei.unipd.it
% 
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program in the file "gpl-license-3.0.txt".  
%
%     If not, see <http://www.gnu.org/licenses/>.
%% 
    
function [angle_vec]=get_angle_vector(S, x_center, y_center, No_Sensors)
%get_angle_vector  
% -- Function File:  

%     Compute the angle of each sensor computed through its vector from the
%     sink and the positive x axis
%
%     Input:
%     S             : Location of each Sensor
%     x_center      : center x-coordinate
%     y_center      : center y-coordinate
%     No_Sensors    : # of sensors
%
%     Output:
%     angle_vec     : Angle of each sensor


center = [x_center,y_center];
v2=[10,0];

for i = 1: No_Sensors
    v1=S(i,:)-center;
    ang = atan2(v1(1)*v2(2)-v2(1)*v1(2),v1(1)*v2(1)+v1(2)*v2(2));
    angle_vec(i) = floor(mod(-180/pi * ang, 360));
end

end
