
%%
%     ---------------------------------------------------------------------
%     Covariogram-based CS data gathering & compression framework
%     ---------------------------------------------------------------------
%     Performance evaluation framework including: 
%               - generation of correlated spatio-temporal signals
%               - scheduling, sampling & data gathering
%               - signal compression (at the sensors) & reconstruction (at the sink)
%
%     Copyright (C) 2016 Mohsen Hooshmand, Davide Zordan, Michele Rossi
%     Department of Information Engineering, University of Padova, Italy
%     Email: {hooshmand, zordan, rossi}@dei.unipd.it
% 
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program in the file "gpl-license-3.0.txt".  
%
%     If not, see <http://www.gnu.org/licenses/>.
%% 
    
function [route_matrix, No_of_hops_HC, No_of_hops] = get_hops_vector( NoS, NeiWithSink, Belonging, Nei_mat, Cid, clustering)
%get_hops_vector  
% -- Function File:  

%     Network Nodes deployment
%
%     Input:
%     Nei_mat       : 0-1 radio neighbor matrix
%     NoS           : # of sensors
%     NeiWithSink   : 0-1 vector sensors neighbor with sink
%     Cid           : Location of each Sensor
%     clustering    :  logical variable to route considering cluster or not
%     to
%
%     Output:
%     route_matrix  : route matrix
%     No_of_hops_HC : if cluster members send data to the cluster-head
%     No_of_hops    : hop # of each sensor


Connected= zeros(NoS,1); % connected to the lan
No_of_hops=zeros(NoS,1);
% route to sink's neibors
without_nei_sensors=find(sum(Nei_mat)==0);
route_matrix=zeros(NoS+1, NoS+1);
[R C] = size(without_nei_sensors);
for i=1:C
    Belonging(without_nei_sensors(i))=1; %  0: not assigned, 1: dead, 2: assigned, 3: HC
    Connected(without_nei_sensors(i))=10;
end

for i=1:NoS
    if(NeiWithSink(i)==1)
        Connected(i)=1;
        route_matrix(i,NoS+1)=1;
        route_matrix(NoS+1,i)=1;
        if(Belonging(i)==3)
            Connected(i)=3;
        end
        No_of_hops(i)=1;
    end
end

%finiding complete route to each sensor
while(min(Connected)<1)
    for i=1:NoS
        if(Connected(i)>=1 && Belonging(i)>1)
            for j=1:NoS
                if(Nei_mat(i,j)==1 && Connected(j)==0)
                    Connected(j)=1;
                    if(Belonging(j)==3)
                        Connected(j)=3;
                    end
                    route_matrix(i,j)=1;
                    route_matrix(j,i)=1;
                    No_of_hops(j)=No_of_hops(i)+1;
                end
            end
        end
    end
end

No_of_hops_HC = No_of_hops;

% if clustering
    %Sum=sum(Connected>=1);
    % finiding  routes From HCs to cluster members
    % route to HCs' neighbors
if(clustering)    
    for i= 1 : NoS
        if(Belonging(i)==3)
            for j=1:NoS
                if(Nei_mat(i,j)==1 && Connected(j)<=1 && Cid(i)==Cid(j))
                    Connected(j)=2;
                    No_of_hops_HC(j)=1;
                end
            end
        end
    end



    while(min(Connected)==1) %(sum(Connected>1)<(NoS-size(without_nei_sensors))) %(min(Connected)==1) % || min(max(Nei_mat))>0)
        for i=1:NoS
            if(Connected(i)>=2)
                for j=1:NoS
                    if(Nei_mat(i,j)==1 && Connected(j)==1)% && Cid(i)==Cid(j))
                        No_of_hops_HC(j)=No_of_hops_HC(i)+1;
                        Connected(j)=2;
                        Sum2=sum(Connected>1);
                        if(Cid(i)~=Cid(j))
                            Cid(i)=Cid(j);
                        end
                    end
                end
            end
        end
    end
end


end
