
%%
%     ---------------------------------------------------------------------
%     Covariogram-based CS data gathering & compression framework
%     ---------------------------------------------------------------------
%     Performance evaluation framework including: 
%               - generation of correlated spatio-temporal signals
%               - scheduling, sampling & data gathering
%               - signal compression (at the sensors) & reconstruction (at the sink)
%
%     Copyright (C) 2016 Mohsen Hooshmand, Davide Zordan, Michele Rossi
%     Department of Information Engineering, University of Padova, Italy
%     Email: {hooshmand, zordan, rossi}@dei.unipd.it
% 
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program in the file "gpl-license-3.0.txt".  
%
%     If not, see <http://www.gnu.org/licenses/>.
%% 
    
function [ Nei_mat, NeiWithSink, Sorted_nei, IX, Dist_mat ] = get_neig_matrix( SoF_X, SoF_Y, NoS, TR, S )
%get_neig_matrix  
% -- Function File:  

%     Generating Distance matrix and Geographical Neighbour Matrix
%
%     Input:
%     SoF_X         : Size of the Field
%     SoF_Y         : Size of the Field
%     NoS           : # of sensors
%     TR            : radius of transmission Range
%     S             : Location of each Sensor
%
%     Output:
%     Nei_mat       : 0-1 radio neighbor matrix
%     NeiWithSink   : 0-1 vector sensors neighbor with sink
%     IX            : sensors' sorting based on highest number of neighbors 
%     Dist_mat      : Distance among sensors matrix


Dist_mat=zeros(NoS);
DistToSink=zeros(NoS,1);
NeiWithSink=zeros(NoS,1);
Nei_mat=zeros(NoS);

 sink_x = floor(SoF_X/2);
 sink_y = floor(SoF_Y/2);

for i=1:NoS
    for j=i+1:NoS
        Dist_mat(i,j)= sqrt((S(i,1)-S(j,1))^2+(S(i,2)-S(j,2))^2);
        Dist_mat(j,i)= Dist_mat(i,j);
        if(Dist_mat(i,j)<=TR)    
            Nei_mat(i,j)= 1;
            Nei_mat(j,i)= 1;
        end
    end
    % Finding Neighbours of the sink
    DistToSink(i)= sqrt((S(i,1)-sink_x)^2+(S(i,2)-sink_y)^2);
    if(DistToSink(i)<=TR)
        NeiWithSink(i)=1;
    end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calculationg the Neighbours of each sensor
NoNei=sum(Nei_mat);
% Sorting sensors based on the number of geographica neighbors
[Sorted_nei, IX]=sort(NoNei,'descend');
end
