
%%
%     ---------------------------------------------------------------------
%     Covariogram-based CS data gathering & compression framework
%     ---------------------------------------------------------------------
%     Performance evaluation framework including: 
%               - generation of correlated spatio-temporal signals
%               - scheduling, sampling & data gathering
%               - signal compression (at the sensors) & reconstruction (at the sink)
%
%     Copyright (C) 2016 Mohsen Hooshmand, Davide Zordan, Michele Rossi
%     Department of Information Engineering, University of Padova, Italy
%     Email: {hooshmand, zordan, rossi}@dei.unipd.it
% 
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program in the file "gpl-license-3.0.txt".  
%
%     If not, see <http://www.gnu.org/licenses/>.
%% 
    
function [y] = ltc_decoder(model,n,N)
% Input:
% -----------------------------------
% N      Number of Samples to Predict
% n      Number Segments (Valid) in Model
%
% model  Segments To Reconstruct with this format:
% 
%        2 Cols: Points Defining Segments. 
%                Each Segment is composed by 2 Sucessive Points 
%                PA=(x(i),y(i)) PB=(x(i+1),y(i+1)) 
%        
%        X(1)  Y(1)   
%        X(2)  Y(2)
%        ...   ...  
%        X(n)  Y(n)
%
%
% Ouput:
% -----------------------------------
%
% y      Row Vector with n Predicted Samples


% Reconstruction Values
y=zeros(1,N);


% Prediction Interpolation
% -------------------------------------

% Segment Index
k=1;

if(n==1)

    y(1)=model(1,2);
    
else

    while(k<n)

        % Take Actual Model
        x1=model(k,1);
        x2=model(k+1,1);
        y1=model(k,2);
        y2=model(k+1,2);

        % Compute Slope
        m=(y2-y1)/(x2-x1);

        i=x1;
        while(i<=x2)
            % Interpolate Point Corresponding to Index i Over Model Segment k
            y(i)=y1+m*(i-x1);
            i=i+1;
        end;    

        k=k+1;

    end    

end 
