%%
%     ---------------------------------------------------------------------
%     Covariogram-based CS data gathering & compression framework
%     ---------------------------------------------------------------------
%     Performance evaluation framework including: 
%               - generation of correlated spatio-temporal signals
%               - scheduling, sampling & data gathering
%               - signal compression (at the sensors) & reconstruction (at the sink)
%
%     Copyright (C) 2016 Mohsen Hooshmand, Davide Zordan, Michele Rossi
%     Department of Information Engineering, University of Padova, Italy
%     Email: {hooshmand, zordan, rossi}@dei.unipd.it
% 
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program in the file "gpl-license-3.0.txt".  
%
%     If not, see <http://www.gnu.org/licenses/>.
%% 
    

function [ S, A, Nei_mat, NeiWithSink, IX, Dist_mat ] = network_deployment( SoF_X, SoF_Y, NoS, synt_data, T, TR  )
%network_deployment  
% -- Function File:  

%     Network Nodes deployment
%
%     Input:
%     SoF_X         : Size of the Field
%     SoF_Y         : Size of the Field
%     NoS           : # of sensors
%     synt_data     : generated signal
%     T             : number of time instants within each round cycle
%     TR            : radius of transmission Range
%
%     Output:
%     S             : Location of each Sensor
%     A             : signal for random location S ---synt_dim*synt_dim*T
%     Nei_mat       : 0-1 radio neighbor matrix
%     NeiWithSink   : 0-1 vector sensors neighbor with sink
%     IX            : sensors' sorting based on highest number of neighbors 
%     Dist_mat      : Distance among sensors matrix


not_connected = true;
while(not_connected)
    S=randi(SoF_X,NoS,2);    
    A = signal_extractor(synt_data, T, NoS, S);       
    [ Nei_mat, NeiWithSink, ~, IX, Dist_mat ] = get_neig_matrix(  SoF_X, SoF_Y, NoS, TR, S );
    adjmatrix = Nei_mat+eye(NoS);
    [p,q,f,s] = dmperm(adjmatrix);
    [R, Cols] = size(p(f(1):f(2)-1));
    if  Cols==NoS
        not_connected = false;
    end
end

end

