/*
*     ---------------------------------------------------------------------
*     Covariogram-based CS data gathering & compression framework
*     ---------------------------------------------------------------------
*     Performance evaluation framework including:
*               - generation of correlated spatio-temporal signals
*               - scheduling, sampling & data gathering
*               - signal compression (at the sensors) & reconstruction (at the sink)
*
*     Copyright (C) 2016 Mohsen Hooshmand, Davide Zordan, Michele Rossi
*     Department of Information Engineering, University of Padova, Italy
*     Email: {hooshmand, zordan, rossi}@dei.unipd.it
*
*     This program is free software: you can redistribute it and/or modify
*     it under the terms of the GNU General Public License as published by
*     the Free Software Foundation, either version 3 of the License, or
*     (at your option) any later version.
*
*     This program is distributed in the hope that it will be useful,
*     but WITHOUT ANY WARRANTY; without even the implied warranty of
*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*     GNU General Public License for more details.
*
*     You should have received a copy of the GNU General Public License
*     along with this program in the file "gpl-license-3.0.txt".
*
*     If not, see <http://www.gnu.org/licenses/>.
*/
#include <iostream>
#include <math.h>
#include "mex.h"

using namespace std;

// MAIN ROUTINE
// Sigma=sigmaBuildCppVar(NoS,distances,lngDistVec,distance_vector,coef,std_dev);
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    int NoS,lngDistVec;
    double *output;
    double *distances;
    double *distance_vector;
    double *coef;
    double *std_dev;
    
    // get input
    if( nrhs!=6)
    {
       cout<<"Hello World!";
    }
    else
    {
        NoS=(int) mxGetScalar(prhs[0]);
        distances=mxGetPr(prhs[1]);
        lngDistVec=(int) mxGetScalar(prhs[2]);
        distance_vector=mxGetPr(prhs[3]);
        coef=mxGetPr(prhs[4]);
        std_dev=mxGetPr(prhs[5]);
    }
    
    
    double tmpVal,tmpDif;
    int tmpIndex;
    double *tmpVect = new double [NoS];
    
    // prepare output
    plhs[0]=mxCreateDoubleMatrix(NoS, NoS, mxREAL);
    output=mxGetPr(plhs[0]);
    
    for(int i=0; i<NoS; i++)
    {
        for(int j=0; j<NoS; j++)
        {
            tmpIndex=0;
            tmpVal=distances[i+NoS*j];
            tmpDif=abs(distance_vector[0] - tmpVal);
            for(int k=1; k<lngDistVec; k++)
            {
                if( abs(distance_vector[k] - tmpVal) < tmpDif)
                {
                    tmpIndex=k;
                    tmpDif=abs(distance_vector[k] - tmpVal);
                }
            }
            output[i+NoS*j]+=coef[0]*exp(coef[1]*distance_vector[tmpIndex]);//*std_dev[i]*std_dev[j];
        }
    }
    return;
}

























