%%
%     ---------------------------------------------------------------------
%     Covariogram-based CS data gathering & compression framework
%     ---------------------------------------------------------------------
%     Performance evaluation framework including: 
%               - generation of correlated spatio-temporal signals
%               - scheduling, sampling & data gathering
%               - signal compression (at the sensors) & reconstruction (at the sink)
%
%     Copyright (C) 2016 Mohsen Hooshmand, Davide Zordan, Michele Rossi
%     Department of Information Engineering, University of Padova, Italy
%     Email: {hooshmand, zordan, rossi}@dei.unipd.it
% 
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program in the file "gpl-license-3.0.txt".  
%
%     If not, see <http://www.gnu.org/licenses/>.
%% 
    

function [ C ] = spatial_correlation_model(corr_model, X, Y, ros, alpha )
%spatial_correlation_model  
% -- Function File:  

%     Spatioal correaltion
%
%     Input:
%     corr_model    : model type 
%     X             : normalized period length
%     Y             : energy required per each cycle opertaion
%     ros           : spatial correlation value
%     alpha         : scaling factor
%
%     Output:
%     C             : spatial correlation matrix


    switch corr_model
        case 'b'
            % Bell shape (default mult. param 0.3)
            % s = 0.00001+10^(i-1);
            C=exp(-(X.^2+Y.^2)/(ros*alpha.^2));
            
        case 'pe'
            % Power Exponential Model
            zeta=f1p.b*alpha;    % correlation length
            nu=f1p.c;            % order of the function
            C=(exp(-(sqrt(X.^2 + Y.^2)./zeta).^nu));
            
        case 'rq'
            % Rational Quadratic Model
            zeta=f2p.b*alpha;    % correlation length
            nu=f2p.c;            % order of the function
            C=1./(1+(20.^(1/nu)-1).*(sqrt(X.^2 + Y.^2)./zeta).^2).^nu;
            
        otherwise
            display('Unknow correlation model. Setting default C=exp(-(X.^2+Y.^2)/(0.3*alpha.^2))');
            C=exp(-(X.^2+Y.^2)/(0.3*alpha.^2));
    end

end

