
%%
%     ---------------------------------------------------------------------
%     Covariogram-based CS data gathering & compression framework
%     ---------------------------------------------------------------------
%     Performance evaluation framework including: 
%               - generation of correlated spatio-temporal signals
%               - scheduling, sampling & data gathering
%               - signal compression (at the sensors) & reconstruction (at the sink)
%
%     Copyright (C) 2016 Mohsen Hooshmand, Davide Zordan, Michele Rossi
%     Department of Information Engineering, University of Padova, Italy
%     Email: {hooshmand, zordan, rossi}@dei.unipd.it
% 
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program in the file "gpl-license-3.0.txt".  
%
%     If not, see <http://www.gnu.org/licenses/>.
%% 
    
function [time_table] = time_table_for_tdma_weight_update_conditional_based(counter, NoS, T, initial_training, cov_function)
%time_table_for_tdma_weight_update_conditional_based  
% -- Function File:  

%     Time Table for a round run
%
%     Input:
%     counter                  : counter/10 is the percentage of sensors
%     that transmit in each time step
%     NoS                      : # of sensors
%     T                        : time instatns in each round cycle
%     initial_training         : Intial tranting time that all sensor send
%     their readings to the sink
%     cov_function             : covariance among sensors based on the
%     initial steps

%     Output:
%     time_table               : Time Table for all sensors for ach round
%     cycle

k = 1;
sorted = zeros(NoS,1);
NoSbM=ceil(counter/10*NoS);
time_table=zeros(NoS,T);
time_slots=ceil(NoS/NoSbM);
subsets=zeros(NoSbM,time_slots);
cov_function1=cov_function;
% subset_number = 1;
v = 1:NoS;
% k = time_slots;
% k = 1;
% subset_number = nchoosek(NoS,k);
subsets_for_eval = nchoosek(v,k);
subset_selected = zeros(size(subsets_for_eval,1),1);


set_nu = floor(NoS/k);
sensor_selected = zeros(NoS,1);
idx = 1;
for enum = 1 : set_nu
    max_set=-500000000;
    for i = 1 : size(subsets_for_eval,1)
        if(subset_selected(i)~=1)           
%         if(sum(sensor_selected(subsets_for_eval(i,:))~=1)==k)           
%             aa=subsets_for_eval(i,:)
            evaluation_mat = cov_function1(:,subsets_for_eval(i,:))*(inv(cov_function1(subsets_for_eval(i,:),subsets_for_eval(i,:))))*cov_function1(subsets_for_eval(i,:),:);
            diagonal = diag(evaluation_mat);
            sum_over_diag = sum(abs(diagonal));
            if(sum_over_diag > max_set && sum(sensor_selected(squeeze(subsets_for_eval(i,:))))==0)
                max_set = sum_over_diag;
                max_idx = i;
            end
        end
    end
    if(subset_selected(max_idx)~=1)        
        subset_selected(max_idx) = 1;
        sensor_selected(squeeze(subsets_for_eval(max_idx,:)))=1;
        sorted(idx:idx+k-1) = subsets_for_eval(max_idx,:);
        idx = idx + k;
        cov_function1 = cov_function1 - cov_function1(:,subsets_for_eval(max_idx,:))*(inv(cov_function1(subsets_for_eval(max_idx,:),subsets_for_eval(max_idx,:))))*cov_function1(subsets_for_eval(max_idx,:),:);
        cov_function1(isnan(cov_function1)) = 0;
    end
end
sorted(idx:end)=find(sensor_selected==0);
I = sorted;
if(sorted==0)
    display('not complete')
end

for i= 1: time_slots
    enum=1;
    l=1;
    enumerature=zeros(NoS,1);
    for j=1:NoSbM
        subsets(j,i)=I(j);
    end
end

for i= 1: time_slots
    enum=1;
    l=1;
    enumerature=zeros(NoS,1);
    for j=1:NoSbM
        if(sum((I(enum)==subsets(:,i)))==0 && enum<=NoS)
            if((enum+i-1)==NoS)
                idx=enum+i-1;
            else
                idx=rem((enum+i-1),NoS);
            end
            subsets(l,i)=I(idx);
            l=l+1;
            enumerature(I(idx))=1;
            enum=enum+time_slots;
            if(enum>=NoS)
                enum=i+1;
            end
        end
    end
    j=1;
    l=sum(enumerature)+1;
    while(sum(enumerature)<NoSbM && j<=NoS)
        if(enumerature(I(j))==0)
            subsets(l,i)=I(j);
            enumerature(I(j))=1;
			l=l+1;
        end
        j=j+1;
    end
end

ml=floor(T/time_slots);
zero_one_matrix=zeros(NoS,time_slots);
for i=1:time_slots
    for j=1:NoSbM
        zero_one_matrix(subsets(j,i),i)=1;
    end
end
% subsets
B = repmat(zero_one_matrix,1,ml);
time_table(:,1:size(B,2))=B;
time_table(:,size(B,2)+1:end)=B(:,1:size(time_table,2)-size(B,2));

time_table(:,1:initial_training)=1;
    
end
